/**
 * File:    Segway.h
 *
 * Author:  David Zemon
 * Project: Segway
 */

/* Software Diagram
 *
 * Cog 0: Main loop
 *
 * Cog 1: SPI assembly cog
 */

#ifndef SEGWAY_H_
#define SEGWAY_H_

//#include <stdio.h>
#include "Segway_globals.h"
#include "Segway_backend.h"
#include <hd44780.h>
#include <stdlib.h>

#ifndef DEBUG_VERBOSE
#define LOOP_CNT_MILLIS			300					// Number of milliseconds in each main loop
#else
#define LOOP_CNT_MILLIS			300
#endif

// Motor pins
#define LEFT_MOTOR_PIN			BIT_12
#define RIGHT_MOTOR_PIN			BIT_14

// Tuning constants
#define LEAN_NULL_ZONE			5					// Range of non-response for the lean angle from zero in degrees
#define ACCEL_DIVISOR			1					// Adjusts the sensitivity of the accelerometer
#define MAX_LEAN				20					// Max allowable angle reading for the platform to lean
#define MAX_POWER				100					// Maximum output from PID loop
#define LOWEST_PWM 				50					// Lowest PWM output value needed since motors do not start spinning at PWM=1
#define LEAN_SHUTOFF			30					// Lean angle at which the Segway will stop the motors for safety
#define TURN_DIVISOR			1					// Adjusts the sensitivity of the turn potentiometer
#define MAX_DIFFERENTIAL		50					// Max difference between wheel outputs when turning

// PID Variables
#define KP						(MAX_POWER - MAX_TURN_POWER) / MAX_LEAN
#define KD						0.3
#define KI						0.1

/*****************************
 *** Function Declarations ***
 *****************************/
void setup (PWMDualMotors *motors, uint32_t *readAngleCogID);

uint8_t assignOutputs (const int8_t leanAngle, const int8_t turnAngle,
		int16_t *leftOut, int16_t *rightOut);

int16_t runPID (int8_t leanAngle);

#endif /* SEGWAY_H_ */
